#!/usr/bin/perl -w

use CGI qw/:standard *blockquote *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e) = @_;
    "<img src=\"$e.png\" alt=\"$e\" align=\"middle\">";
}

$NATURAL = '<img src="natural.png">';  # without align

param('a:min', 1);
param('a:max', 50);
param('b:min', -50);
param('b:max', 50);
param('x:min', -50);
param('x:max', 50);

my $dim_min     = 1;
my $dim_default = 3;
my $dim_max     = 10;
my $dim = param('dim');
unless (defined($dim) && $dim_min <= $dim && $dim <= $dim_max) {
    $dim = $dim_default;
    param('dim', $dim);
}

sub change_dim {
    $dim = int(param('newdim'));
    unless ($dim_min <= $dim && $dim <= $dim_max) {
	$dim = $dim_default;
    }
    param('dim', $dim);
}

sub print_header {
print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Quadratic M#-Convex',
	       -head=>[meta( { '-http_equiv' => 'Content-style-type',
			       '-content'    => 'text/html' } ),
		       meta( { '-http_equiv' => 'Content-script-type',
			       '-content'    => 'text/javascript' } )],
	       -onload=>'check()'),
    ln, <<EOF,
<script type="text/javascript">
<!--
function check() {
    var p2 = document.p2;
    var outrange = false;
    var dummy = 0;
    var list = {'a':dummy, 'b':dummy, 'x':dummy};
    for (var v in list) {
	var min = ~~p2.elements[v+':min'].value;
	var max = ~~p2.elements[v+':max'].value;
	for (i=0; i<$dim; i++) {
	    var e = p2.elements[v+':'+i];
	    e.value = ~~e.value;  // parseInt(e.value);
	    if (min <= e.value && e.value <= max) {
		e.style.backgroundColor = 'white';
	    } else {
		e.style.backgroundColor = 'pink';
		outrange = true;
	    }
	}
    }
    for (i=1; i<$dim; i++) {
	if (p2.elements['a:'+(i-1)].value - 0 >=
	    p2.elements['a:'+i].value - 0) {
	    p2.elements['aa:'+i].style.backgroundColor = 'white';
	} else {
	    p2.elements['aa:'+i].style.backgroundColor = 'pink';
	    outrange = true;
	}
    }
    if (outrange) {
	document.getElementById('message').innerHTML
	    = '<font color="red">Minimality is not guaranteed.</font>';
    } else {
	document.getElementById('message').innerHTML = '';
    }

    for (i=1; i<$dim; i++) {
	for (j=0; j<$dim; j++) {
	    if (i<j) {
		p2.elements['a:'+i+':'+j].value = p2.elements['a:'+j].value;
	    } else {
		p2.elements['a:'+i+':'+j].value = p2.elements['a:'+i].value;
	    }
	}
    }
}

function popup_window(form) {
    obj = window.open("", "popup", "width=400, height=400, scrollbars=yes, menubar=no, location=no, directories=no");
    form.target = "popup";
    form.action = "./odicon.cgi";
    obj.focus();
    form.submit();
}
// http://www.phppro.jp/qa/609
// -->
</script>
EOF
    a({href=>"./"},'DCP HOME'), ln,
    h1('Quadratic M<img src="natural.png" width=14>-Convex Minimization'), ln,
    start_blockquote, ln;
}

sub set_param {
    my ($var, $min, $max) = @_;
    my $val = $min + int(rand($max-$min+1));
    param($var, $val);
}

sub init_param {
    my ($i, $j);

    foreach $e ('a', 'b', 'x') {
	my $min = param("$e:min");
	my $max = param("$e:max");
	for ($i=0; $i<$dim; $i++) {
	    set_param("$e:$i", $min, $max);
	}
    }

    my @as = ();
    for ($i=0; $i<$dim; $i++) { push @as, param("a:$i"); }
    @as = sort { $b <=> $a } @as;
    for ($i=0; $i<$dim; $i++) { param("a:$i", $as[$i]); }
}

sub print_html_table {
    print 'For a symmetric matrix ',
    image('A'), ' with ', ln;
    print p,image('aij'), ln;
    print p, 'for some ',
    image('w1wn'), ', the associated quadratic form  ',
    image('quadform'), 
    " is an M$NATURAL-convex function [1,2]. ",
    'For n=4, such a matrix looks like', ln;
    print p, image('defA');
    print p, "Here we consider a quadratic M$NATURAL-convex function:",
    ln;

    print p, image('objfunc-quadM');
    print p, 'In this web application, you can choose the dimension n from 1 to 10.', ln;

    print start_form, start_div({-align=>'center'}), ln;
    @index = (1..10);
    print p, 'dimension n = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change'), ln, end_form, ln;

    print p, 'You can input parameters a', Sub(em('ij')),
    ' and b', Sub(em('i')),
    ' in the ranges 0 &lt; a', Sub(em('ij')), '&lt; 50',
    ' and -50 &lt; b', Sub(em('i')), '&lt; 50.',
    ' If the input does not satisfy the condition ', image('w1wn'),
    ', the unsatisfactory condition turns red.';
    #', b', Sub(em('i')),', c', Sub(em('i')),
    print p, 'You can also input an initial solution x.';
  
    print start_form(-name=>'p2',
		     -action=>'javascript:popup_window(document.p2)'),
    hidden('dim'), ln;

    foreach $e ('a') {
	print br, hidden("$e:min"), hidden("$e:max"),
	start_table({-border=>'1'}), ln,
	Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'},
				  [ $e, 1 .. $dim ])]);
	for ($i=0; $i<$dim; $i++) {
	    my @td = ($i+1);
	    for ($j=0; $j<$dim; $j++) {
		my $element = '';
		if ($i == 0) {
		    if ($j > 0) {
			$element .= textfield(-name=>"$e$e:$j", -size=>1,
					      -default=>'>=', -disabled);
		    }
		    $element .= textfield(-name=>"$e:$j", -size=>3,
					  -default=>param("$e:$j"),
					  -onMouseOut=>"check()",
					  -onBlur=>"check()");
		} else {
		    $element .= textfield(-name=>"$e:$i:$j", -size=>3,
					  -disabled);
		}
		push @td, $element;
	    }
	    print Tr({-align=>'right'}, [td([@td])]), ln;
	}
	print end_table, ln;
    }

    print br, start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'},
				    [ '', 1 .. $dim ])]);
    foreach $e ('b', 'x') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3,
				-default=>param("$e:$j"),
				-onMouseOut=>"check()", -OnBlur=>"check()");
	}
	print Tr({-align=>'right'}, [td([@td])]),
	hidden("$e:min"), hidden("$e:max"), ln;
    }
    print end_table, ln;

    print p, '(Note: "Reset" button resets your inputs to default values.)', ln;
    print p, start_table(),Tr(td({-id=>'message'}, '')),end_table,
    submit('Minimize'), reset({-onMouseOut=>"check()", -onBlur=>"check()", -name=>'Reset'}), ln;
    print end_div, end_form, ln;
}

sub print_reference {
    print <<EOF;
<hr width="70%">
<p> This web application minimizes f(x) using
<a href="http://www.misojiro.t.u-tokyo.ac.jp/~tutimura/odicon/index.en.html">ODICON</a>. 

<p> [1] K. Murota (2001): "Discrete Convex Analysis---An Introduction (in Japanese),"
Kyoritsu Publishing Company, Tokyo. Section 5.6.

<p> [2] K. Murota (2003):
"Discrete Convex Analysis," SIAM. Section 6.3.
EOF
# <p> [3] K. Murota,
# Recent developments in discrete convex analysis,
# in: W. Cook, L. Lovasz and J. Vygen, eds.,
# Research Trends in Combinatorial Optimization, Bonn 2008,
# Springer-Verlag, Berlin, (2009), Chapter 11, 219--260.
}

change_dim() if param('Change');
init_param();
print_header();
print_html_table();
print_reference();

print end_blockquote, end_html, ln;
